@@ -1,8 +1,8 @@
 <?php
 /**
- * $Horde: kronolith/deleventaction.php,v 1.22.10.2 2005/03/31 04:16:19 chuck Exp $
+ * $Horde: kronolith/deleventaction.php,v 1.22.10.6 2007/01/02 13:55:04 jan Exp $
  *
- * Copyright 1999-2005 Charles J. Hagenbuch <chuck@horde.org>
+ * Copyright 1999-2007 Charles J. Hagenbuch <chuck@horde.org>
  *
  * See the enclosed file COPYING for license information (GPL).  If you
  * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
@@ -14,7 +14,7 @@ require_once KRONOLITH_BASE . '/lib/base.php';
 $kronolith->open(Util::getFormData('calendar'));
 if ($eventID = Util::getFormData('eventID')) {
     $event = &$kronolith->getEvent($eventID);
-    if (!$event) {
+    if (is_a($event, 'PEAR_Error')) {
         if (($url = Util::getFormData('url')) === null) {
             $url = Horde::applicationUrl($prefs->getValue('defaultview') . '.php', true);
         }
