@@ -3,7 +3,7 @@
  * The Kronolith_Driver_sql:: class implements the Kronolith_Driver
  * API for a SQL backend.
  *
- * $Horde: kronolith/lib/Driver/sql.php,v 1.136.2.13 2005/09/25 15:22:12 jan Exp $
+ * $Horde: kronolith/lib/Driver/sql.php,v 1.136.2.27 2007/03/08 13:52:35 jan Exp $
  *
  * @author  Luc Saillard <luc.saillard@fr.alcove.com>
  * @author  Chuck Hagenbuch <chuck@horde.org>
@@ -81,12 +81,14 @@ class Kronolith_Driver_sql extends Kronolith_Driver {
 
     function search($query)
     {
-        /* build SQL conditions based on the query string */
+        require_once 'Horde/SQL.php';
+
+        // Build SQL conditions based on the query string.
         $cond = '((';
         $values = array();
 
         if (!empty($query->title)) {
-            $binds = Horde_SQL::buildClause($this->_db, 'event_title', 'LIKE', $query->title, true);
+            $binds = Horde_SQL::buildClause($this->_db, 'event_title', 'LIKE', $this->convertToDriver($query->title), true);
             if (is_array($binds)) {
                 $cond .= $binds[0] . ' AND ';
                 $values = array_merge($values, $binds[1]);
@@ -95,7 +97,7 @@ class Kronolith_Driver_sql extends Kronolith_Driver {
             }
         }
         if (!empty($query->location)) {
-            $binds = Horde_SQL::buildClause($this->_db, 'event_location', 'LIKE', $query->location, true);
+            $binds = Horde_SQL::buildClause($this->_db, 'event_location', 'LIKE', $this->convertToDriver($query->location), true);
             if (is_array($binds)) {
                 $cond .= $binds[0] . ' AND ';
                 $values = array_merge($values, $binds[1]);
@@ -104,7 +106,7 @@ class Kronolith_Driver_sql extends Kronolith_Driver {
             }
         }
         if (!empty($query->description)) {
-            $binds = Horde_SQL::buildClause($this->_db, 'event_description', 'LIKE', $query->description, true);
+            $binds = Horde_SQL::buildClause($this->_db, 'event_description', 'LIKE', $this->convertToDriver($query->description), true);
             if (is_array($binds)) {
                 $cond .= $binds[0] . ' AND ';
                 $values = array_merge($values, $binds[1]);
@@ -113,7 +115,7 @@ class Kronolith_Driver_sql extends Kronolith_Driver {
             }
         }
         if (isset($query->category)) {
-            $binds = Horde_SQL::buildClause($this->_db, 'event_category', '=', $query->category, true);
+            $binds = Horde_SQL::buildClause($this->_db, 'event_category', '=', $this->convertToDriver($query->category), true);
             if (is_array($binds)) {
                 $cond .= $binds[0] . ' AND ';
                 $values = array_merge($values, $binds[1]);
@@ -159,6 +161,55 @@ class Kronolith_Driver_sql extends Kronolith_Driver {
         return $events;
     }
 
+    /**
+     * Checks if the event's UID already exists and returns all event
+     * ids with that UID.
+     *
+     * @param string $uid          The event's uid.
+     * @param string $calendar_id  Calendar to search in.
+     *
+     * @return string|boolean  Returns a string with event_id or false if
+     *                         not found.
+     */
+    function exists($uid, $calendar_id = null)
+    {
+        $query = 'SELECT event_id  FROM ' . $this->_params['table'] . ' WHERE event_uid = ?';
+        $values = array($uid);
+
+        if (!is_null($calendar_id)) {
+            $query .= ' AND calendar_id = ?';
+            $values[] = $calendar_id;
+        }
+
+        // Log the query at a DEBUG log level.
+        Horde::logMessage(sprintf('SQL event fetch by %s: query = "%s"',
+                                  Auth::getAuth(), $query),
+                          __FILE__, __LINE__, PEAR_LOG_DEBUG);
+
+        $event = &$this->_db->getRow($query, $values, DB_FETCHMODE_ASSOC);
+        if (is_a($event, 'PEAR_Error')) {
+            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
+            return $event;
+        }
+
+        if ($event) {
+            return $event['event_id'];
+        } else {
+            return false;
+        }
+    }
+
+    /**
+     * Lists all events in the time range, optionally restricting
+     * results to only events with alarms.
+     *
+     * @param Horde_Date $startInterval  Start of range date object.
+     * @param Horde_Date $endInterval    End of range data object.
+     * @param boolean $hasAlarm          Only return events with alarms?
+     *                                   Defaults to all events.
+     *
+     * @return array  Events in the given time range.
+     */
     function listEvents($startDate = null, $endDate = null, $hasAlarm = false)
     {
         if (!isset($endDate)) {
@@ -200,7 +251,7 @@ class Kronolith_Driver_sql extends Kronolith_Driver {
      *                conditions.
      */
     function listEventsConditional($startInterval, $endInterval,
-        $conditions = '', $vals = array())
+                                   $conditions = '', $vals = array())
     {
         $q = 'SELECT event_id, event_uid, event_description, event_location,' .
             ' event_status, event_attendees,' .
@@ -233,12 +284,12 @@ class Kronolith_Driver_sql extends Kronolith_Driver {
             ' AND event_recurtype <> ?))';
         array_push($values, $etime, KRONOLITH_RECUR_NONE);
 
-        /* Log the query at a DEBUG log level. */
+        // Log the query at a DEBUG log level.
         Horde::logMessage(sprintf('SQL event list by %s: query = "%s"',
                                   Auth::getAuth(), $q),
                           __FILE__, __LINE__, PEAR_LOG_DEBUG);
 
-        /* Run the query. */
+        // Run the query.
         $qr = $this->_db->query($q, $values);
         if (is_a($qr, 'PEAR_Error')) {
             Horde::logMessage($qr, __FILE__, __LINE__, PEAR_LOG_ERR);
@@ -248,15 +299,35 @@ class Kronolith_Driver_sql extends Kronolith_Driver {
         $events = array();
         $row = $qr->fetchRow(DB_FETCHMODE_ASSOC);
         while ($row && !is_a($row, 'PEAR_Error')) {
+            // If the event did not have a UID before, we need to give
+            // it one.
+            if (empty($row['event_uid'])) {
+                $row['event_uid'] = $this->generateUID();
+
+                // Save the new UID for data integrity.
+                $query = 'UPDATE ' . $this->_params['table'] . ' SET event_uid = ? WHERE event_id = ?';
+                $values = array($row['event_uid'], $row['event_id']);
+
+                // Log the query at a DEBUG log level.
+                Horde::logMessage(sprintf('SQL event uid autocreation for %s: query = "%s"',
+                                          Auth::getAuth(), $query),
+                                  __FILE__, __LINE__, PEAR_LOG_DEBUG);
+
+                $result = $this->_db->query($query, $values);
+                if (is_a($result, 'PEAR_Error')) {
+                    Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
+                }
+            }
+
             // We have all the information we need to create an event
             // object for this event, so go ahead and cache it.
             $this->_cache[$this->_calendar][$row['event_id']] = &new Kronolith_Event_sql($this, $row);
             if ($row['event_recurtype'] == KRONOLITH_RECUR_NONE) {
-                $events[$row['event_id']] = $row['event_id'];
+                $events[$row['event_uid']] = $row['event_id'];
             } else {
                 $next = $this->nextRecurrence($row['event_id'], $startInterval);
                 if ($next && $next->compareDate($endInterval) < 0) {
-                    $events[$row['event_id']] = $row['event_id'];
+                    $events[$row['event_uid']] = $row['event_id'];
                 }
             }
 
@@ -269,7 +340,8 @@ class Kronolith_Driver_sql extends Kronolith_Driver {
     function &getEvent($eventId = null)
     {
         if (is_null($eventId)) {
-            return $event = &new Kronolith_Event_sql($this);
+            $event = &new Kronolith_Event_sql($this);
+            return $event;
         }
 
         if (isset($this->_cache[$this->_calendar][$eventId])) {
@@ -285,7 +357,7 @@ class Kronolith_Driver_sql extends Kronolith_Driver {
             ' FROM ' . $this->_params['table'] . ' WHERE event_id = ? AND calendar_id = ?';
         $values = array($eventId, $this->_calendar);
 
-        /* Log the query at a DEBUG log level. */
+        // Log the query at a DEBUG log level.
         Horde::logMessage(sprintf('SQL event fetch by %s: query = "%s"',
                                   Auth::getAuth(), $query),
                           __FILE__, __LINE__, PEAR_LOG_DEBUG);
@@ -300,11 +372,11 @@ class Kronolith_Driver_sql extends Kronolith_Driver {
             $this->_cache[$this->_calendar][$eventId] = &new Kronolith_Event_sql($this, $event);
             return $this->_cache[$this->_calendar][$eventId];
         } else {
-            return false;
+            return PEAR::raiseError(_("Event not found"));
         }
     }
 
-    function &getByUID($uid)
+    function &getByUID($uid, $getAll = false)
     {
         $this->_connect();
 
@@ -317,24 +389,57 @@ class Kronolith_Driver_sql extends Kronolith_Driver {
             ' FROM ' . $this->_params['table'] . ' WHERE event_uid = ?';
         $values = array($uid);
 
-        /* Log the query at a DEBUG log level. */
+        // Log the query at a DEBUG log level.
         Horde::logMessage(sprintf('SQL event fetch by %s: query = "%s"',
                                   Auth::getAuth(), $query),
                           __FILE__, __LINE__, PEAR_LOG_DEBUG);
 
-        $event = &$this->_db->getRow($query, $values, DB_FETCHMODE_ASSOC);
-        if (is_a($event, 'PEAR_Error')) {
-            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
-            return $event;
+        $events = $this->_db->getAll($query, $values, DB_FETCHMODE_ASSOC);
+        if (is_a($events, 'PEAR_Error')) {
+            Horde::logMessage($events, __FILE__, __LINE__, PEAR_LOG_ERR);
+            return $events;
         }
-
-        if ($event) {
-            $this->open($event['calendar_id']);
-            $this->_cache[$this->_calendar][$event['event_id']] = &new Kronolith_Event_sql($this, $event);
-            return $this->_cache[$this->_calendar][$event['event_id']];
-        } else {
+        if (!count($events)) {
             return PEAR::raiseError($uid . ' not found');
         }
+
+        $eventArray = array();
+        foreach ($events as $event) {
+            $this->open($event['calendar_id']);
+            $this->_cache[$this->_calendar][$event['event_id']] = &new Kronolith_Event_sql($this, $event);
+            $eventArray[] = &$this->_cache[$this->_calendar][$event['event_id']];
+        }
+
+        if ($getAll) {
+            return $eventArray;
+        }
+
+        /* First try the user's own calendars. */
+        $ownerCalendars = Kronolith::listCalendars(true, PERMS_READ);
+        $event = null;
+        foreach ($eventArray as $ev) {
+            if (isset($ownerCalendars[$ev->getCalendar()])) {
+                $event = $ev;
+                break;
+            }
+        }
+
+        /* If not successful, try all calendars the user has access too. */
+        if (empty($event)) {
+            $readableCalendars = Kronolith::listCalendars(false, PERMS_READ);
+            foreach ($eventArray as $ev) {
+                if (isset($readableCalendars[$ev->getCalendar()])) {
+                    $event = $ev;
+                    break;
+                }
+            }
+        }
+
+        if (empty($event)) {
+            $event = $eventArray[0];
+        }
+
+        return $event;
     }
 
     /**
@@ -343,9 +448,9 @@ class Kronolith_Driver_sql extends Kronolith_Driver {
      *
      * @param Kronolith_Event $event  The event to save.
      */
-    function saveEvent($event)
+    function saveEvent(&$event)
     {
-        if ($event->isStored()) {
+        if ($event->isStored() || $event->exists()) {
             $values = array();
 
             $query = 'UPDATE ' . $this->_params['table'] . ' SET ';
@@ -358,7 +463,7 @@ class Kronolith_Driver_sql extends Kronolith_Driver {
             $query .= ' WHERE event_id = ?';
             $values[] = $event->getId();
 
-            /* Log the query at a DEBUG log level. */
+            // Log the query at a DEBUG log level.
             Horde::logMessage(sprintf('SQL event update by %s: query = "%s"',
                                       Auth::getAuth(), $query),
                               __FILE__, __LINE__, PEAR_LOG_DEBUG);
@@ -369,24 +474,32 @@ class Kronolith_Driver_sql extends Kronolith_Driver {
                 return $result;
             }
 
-            /* Log the modification of this item in the history log. */
+            // Log the modification of this item in the history log.
             if ($event->getUID()) {
                 $history = &Horde_History::singleton();
                 $history->log('kronolith:' . $this->_calendar . ':' . $event->getUID(), array('action' => 'modify'), true);
             }
 
+            // Notify users about the changed event.
+            $result = Kronolith::sendNotification($event, 'edit');
+            if (is_a($result, 'PEAR_Error')) {
+                Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
+            }
+
             return $event->getId();
         } else {
             if ($event->getId()) {
                 $id = $event->getId();
             } else {
                 $id = md5(uniqid(mt_rand(), true));
+                $event->setId($id);
             }
 
             if ($event->getUID()) {
                 $uid = $event->getUID();
             } else {
                 $uid = $this->generateUID();
+                $event->setUID($uid);
             }
 
             $query = 'INSERT INTO ' . $this->_params['table'];
@@ -406,7 +519,7 @@ class Kronolith_Driver_sql extends Kronolith_Driver {
 
             $query .= $cols_name . $cols_values;
 
-            /* Log the query at a DEBUG log level. */
+            // Log the query at a DEBUG log level.
             Horde::logMessage(sprintf('SQL event store by %s: query = "%s"',
                                 Auth::getAuth(), $query),
                                 __FILE__, __LINE__, PEAR_LOG_DEBUG);
@@ -417,10 +530,16 @@ class Kronolith_Driver_sql extends Kronolith_Driver {
                 return $result;
             }
 
-            /* Log the creation of this item in the history log. */
+            // Log the creation of this item in the history log.
             $history = &Horde_History::singleton();
             $history->log('kronolith:' . $this->_calendar . ':' . $uid, array('action' => 'add'), true);
 
+            // Notify users about the new event.
+            $result = Kronolith::sendNotification($event, 'add');
+            if (is_a($result, 'PEAR_Error')) {
+                Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
+            }
+
             return $id;
         }
     }
@@ -433,17 +552,17 @@ class Kronolith_Driver_sql extends Kronolith_Driver {
      */
     function move($eventId, $newCalendar)
     {
-        /* Make sure we have a valid database connection. */
+        // Make sure we have a valid database connection.
         $this->_connect();
 
         $query = 'UPDATE ' . $this->_params['table'] . ' SET calendar_id = ? WHERE calendar_id = ? AND event_id = ?';
         $values = array($newCalendar, $this->_calendar, $eventId);
 
-        /* Log the query at a DEBUG log level. */
+        // Log the query at a DEBUG log level.
         Horde::logMessage(sprintf('Kronolith_Driver_sql::move(): %s', $query),
                           __FILE__, __LINE__, PEAR_LOG_DEBUG);
 
-        /* Attempt the move query. */
+        // Attempt the move query.
         $result = $this->_db->query($query, $values);
         if (is_a($result, 'PEAR_Error')) {
             Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
@@ -467,7 +586,7 @@ class Kronolith_Driver_sql extends Kronolith_Driver {
         $query = 'DELETE FROM ' . $this->_params['table'] . ' WHERE calendar_id = ?';
         $values = array($calendar);
 
-        /* Log the query at a DEBUG log level. */
+        // Log the query at a DEBUG log level.
         Horde::logMessage(sprintf('SQL Calender Delete by %s: query = "%s"',
                                   Auth::getAuth(), $query),
                           __FILE__, __LINE__, PEAR_LOG_DEBUG);
@@ -484,13 +603,13 @@ class Kronolith_Driver_sql extends Kronolith_Driver {
      */
     function deleteEvent($eventId)
     {
-        /* Fetch the event for later use. */
+        // Fetch the event for later use.
         $event = &$this->getEvent($eventId);
 
         $query = 'DELETE FROM ' . $this->_params['table'] . ' WHERE event_id = ? AND calendar_id = ?';
         $values = array($eventId, $this->_calendar);
 
-        /* Log the query at a DEBUG log level. */
+        // Log the query at a DEBUG log level.
         Horde::logMessage(sprintf('SQL Event Delete by %s: query = "%s"',
                                   Auth::getAuth(), $query),
                           __FILE__, __LINE__, PEAR_LOG_DEBUG);
@@ -501,12 +620,18 @@ class Kronolith_Driver_sql extends Kronolith_Driver {
             return $result;
         }
 
-        /* Log the deletion of this item in the history log. */
+        // Log the deletion of this item in the history log.
         if ($event->getUID()) {
             $history = &Horde_History::singleton();
             $history->log('kronolith:' . $this->_calendar . ':' . $event->getUID(), array('action' => 'delete'), true);
         }
 
+        // Notify about the deleted event.
+        $result = Kronolith::sendNotification($event, 'delete');
+        if (is_a($result, 'PEAR_Error')) {
+            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
+        }
+
         return true;
     }
 
@@ -534,7 +659,7 @@ class Kronolith_Driver_sql extends Kronolith_Driver {
                 $this->_params['table'] = 'kronolith_events';
             }
 
-            /* Connect to the SQL server using the supplied parameters. */
+            // Connect to the SQL server using the supplied parameters.
             require_once 'DB.php';
             $this->_db = &DB::connect($this->_params,
                                       array('persistent' => !empty($this->_params['persistent'])));
@@ -542,17 +667,24 @@ class Kronolith_Driver_sql extends Kronolith_Driver {
                 Horde::fatal($this->_db, __FILE__, __LINE__);
             }
 
-            $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);
+            // Set DB portability options.
+            switch ($this->_db->phptype) {
+            case 'mssql':
+                $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS | DB_PORTABILITY_RTRIM);
+                break;
+            default:
+                $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);
+            }
 
             $this->_connected = true;
 
-            /* Handle any database specific initialization code to
-             * run. */
+            // Handle any database specific initialization code to
+            // run.
             switch ($this->_db->dbsyntax) {
             case 'oci8':
                 $query = "ALTER SESSION SET NLS_DATE_FORMAT = 'YYYY-MM-DD HH24:MI:SS'";
 
-                /* Log the query at a DEBUG log level. */
+                // Log the query at a DEBUG log level.
                 Horde::logMessage(sprintf('SQL session setup by %s: query = "%s"',
                                           Auth::getAuth(), $query),
                                   __FILE__, __LINE__, PEAR_LOG_DEBUG);
@@ -563,7 +695,7 @@ class Kronolith_Driver_sql extends Kronolith_Driver {
             case 'pgsql':
                 $query = "SET datestyle TO 'iso'";
 
-                /* Log the query at a DEBUG log level. */
+                // Log the query at a DEBUG log level.
                 Horde::logMessage(sprintf('SQL session setup by %s: query = "%s"',
                                           Auth::getAuth(), $query),
                                   __FILE__, __LINE__, PEAR_LOG_DEBUG);
@@ -632,7 +764,10 @@ class Kronolith_Event_sql extends Kronolith_Event {
 
         if (isset($SQLEvent['event_recurenddate'])) {
             $this->recurEnd = &new Horde_Date();
-            list($this->recurEnd->year, $this->recurEnd->month, $this->recurEnd->mday, $this->recurEnd->hour, $this->recurEnd->min, $this->recurEnd->sec) = sscanf($SQLEvent['event_recurenddate'], '%04d-%02d-%02d %02d:%02d:%02d');
+            list($this->recurEnd->year, $this->recurEnd->month, $this->recurEnd->mday) = sscanf($SQLEvent['event_recurenddate'], '%04d-%02d-%02d 00:00:00');
+            $this->recurEnd->hour = 23;
+            $this->recurEnd->min = 59;
+            $this->recurEnd->sec = 59;
         }
 
         $this->title = $driver->convertFromDriver($SQLEvent['event_title']);
@@ -652,7 +787,7 @@ class Kronolith_Event_sql extends Kronolith_Event {
             $this->status = $SQLEvent['event_status'];
         }
         if (isset($SQLEvent['event_attendees'])) {
-            $this->attendees = unserialize($driver->convertFromDriver($SQLEvent['event_attendees']));
+            $this->attendees = array_change_key_case(unserialize($driver->convertFromDriver($SQLEvent['event_attendees'])));
         }
         if (isset($SQLEvent['event_keywords'])) {
             $this->keywords = explode(',', $driver->convertFromDriver($SQLEvent['event_keywords']));
@@ -710,7 +845,7 @@ class Kronolith_Event_sql extends Kronolith_Event {
         $this->_properties['event_recurtype'] = $recur;
         if ($recur != KRONOLITH_RECUR_NONE) {
             $this->_properties['event_recurinterval'] = $this->getRecurInterval();
-            $this->_properties['event_recurenddate'] = sprintf('%04d%02d%02d', $recur_end[0],
+            $this->_properties['event_recurenddate'] = sprintf('%04d-%02d-%02d', $recur_end[0],
                                                                $recur_end[1], $recur_end[2]);
 
             switch ($recur) {
